/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.swing;

import java.awt.*;

import javax.swing.*;


/**
 * just a small rectangle
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.10 $
 */
public class JRectangle extends JComponent
{
   /**
    * Constructor for class JRectangle
    */
   public JRectangle()
   {
      this (5, 5);
   }


   /**
    * Constructor for class JRectangle
    *
    * @param width   No description provided
    * @param height  No description provided
    */
   public JRectangle (int width, int height)
   {
      super();

      Dimension dim = new Dimension (width, height);
      setSize (dim);
      setPreferredSize (dim);

   }


   /**
    * @see   #isOutlined()
    */
   private boolean outlined = false;


   /**
    * @return   true if only a one pixel border is drawn
    */
   public boolean isOutlined()
   {
      return outlined;
   }


   /**
    * @param outlined  true to outline rectangle
    * @see             #isOutlined()
    */
   public void setOutlined (boolean outlined)
   {
      this.outlined = outlined;
   }


   /**
    * @param g  Graphics
    */
   public void paintComponent (Graphics g)
   {
      Color oldColor = g.getColor();
      Rectangle bounds = getBounds();

      // fill rectangle (0,0,radius*2,radius*2) with color
      g.setColor (getForeground());

      if (!isOutlined())
      {
         g.fillRect (0, 0, bounds.width, bounds.height);
      }
      else
      {
         g.drawRect (0, 0, bounds.width - 1, bounds.height - 1);
      }

      g.setColor (oldColor);
   }
}

/*
 * $Log: JRectangle.java,v $
 * Revision 1.10  2004/11/03 10:17:58  lowende
 * Javadoc warnings removed.
 *
 */
