/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.update;

import java.awt.event.ItemListener;
import java.beans.PropertyChangeListener;

import de.uni_paderborn.fujaba.fsa.FSAComboBox;
import de.uni_paderborn.fujaba.fsa.FSAObject;


/**
 * add and handle listener for changes of selected item in combobox
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.12 $
 */
public class ComboBoxUpdater
    extends LogicAndFsaUpdater
{
   /**
    * Constructor for class ComboBoxUpdater
    */
   public ComboBoxUpdater()
   {
      super();
      super.setFsaAttrName ("selectedItem");
   }


   /**
    * Sets the fsaObject attribute of the ComboBoxUpdater object
    *
    * @param object  The new fsaObject value
    * @return        No description provided
    */
   public boolean setFsaObject (FSAObject object)
   {
      if (object == null || object instanceof FSAComboBox)
      {
         return super.setFsaObject (object);
      }
      throw new IllegalArgumentException ("Object must be of instance FSAComboBox");
   }


   /**
    * Sets the fsaAttrName attribute of the ComboBoxUpdater object
    *
    * @param name  The new fsaAttrName value
    * @return      No description provided
    */
   public boolean setFsaAttrName (String name)
   {
      if ("selectedItem".equals (name))
      {
         return false;
      }
      else
      {
         throw new UnsupportedOperationException ("FsaAttrName cannot be changed from " +
            "\"selectedItem\" in ComboBoxUpdater");
      }
   }


   /**
    * Access method for an one to n association.
    */
   protected void addListener()
   {
      if (! (getLogicObject() == null || getLogicAttrName() == null ||
         getFsaObject() == null || getFsaAttrName() == null))
      {
         if (getLogicListener() == null)
         {
            setLogicListener (new LogicToFsaUpdatePCL (this, getFsaAttrName(),
               getFsaObject()));
         }
         ListenerHelper.get().addPropertyChangeListener (getLogicObject(), getLogicAttrName(), (PropertyChangeListener) getLogicListener());

         if (getFsaListener() == null)
         {
            setFsaListener (new FsaToLogicUpdateItemListener (this, getLogicAttrName(),
               getLogicObject()));
         }
          ((FSAComboBox) getFsaObject()).addItemListener ((ItemListener) getFsaListener());
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected void removeListener()
   {
      if (! (getLogicObject() == null || getLogicAttrName() == null ||
         getFsaObject() == null || getFsaAttrName() == null))
      {
         if (getLogicListener() != null)
         {
            ListenerHelper.get().removePropertyChangeListener (getLogicObject(), getLogicAttrName(), (PropertyChangeListener) getLogicListener());
         }

         if (getFsaListener() != null)
         {
             ((FSAComboBox) getFsaObject()).removeItemListener ((ItemListener) getFsaListener());
         }
      }
   }
}

/*
 * $Log: ComboBoxUpdater.java,v $
 * Revision 1.12  2004/11/03 10:18:02  lowende
 * Javadoc warnings removed.
 *
 */
